-- What the NEB module will use when it gets a service status event.
-- It's nice and transaction safe, and its execution plan is helpfully cached after the first run.

CREATE or REPLACE FUNCTION update_service(
	text,text,boolean,boolean,boolean,
	boolean,boolean,boolean,boolean,boolean,
	text,int,boolean,boolean,boolean,
	int,int,text,int,text,
	float,float,int,int,int,
	int,int,boolean,text,int) RETURNS 
integer AS
'       DECLARE
		thisHostName			alias for $1;
		ServiceDescription		alias for $2;
		ActiveChecksEnabled		alias for $3;
		PassiveChecksEnabled		alias for $4;
		EventHandlerEnabled		alias for $5;
		FlapDetectionEnabled		alias for $6;
		NotificationsEnabled		alias for $7;
		FailurePredictionEnabled	alias for $8;
		ProcessPerformanceData		alias for $9;
		ObsessOverService		alias for $10;
		EventHandler			alias for $11;
		ModifiedAttributes		alias for $12;
		HasBeenChecked			alias for $13;
		ShouldBeScheduled		alias for $14;
		IsFlapping			alias for $15;
		SoftState			alias for $16;
		HardState			alias for $17;
		PluginOutput			alias for $18;
		CurrentAttempt			alias for $19;
		CheckCommand			alias for $20;
		CheckExecuteTime		alias for $21;
		CheckLatency			alias for $22;
		LastCheck			alias for $23;
		NextCheck			alias for $24;
		MaxAttempts			alias for $25;
		LastNotification		alias for $26;
		NextNotification		alias for $27;
		NoMoreNotifications		alias for $28;
		PerformanceData			alias for $29;
		CheckType			alias for $30;
     		thisHostID			int;
     		thisServiceID			int;
		c				int;
		states				record;
        BEGIN
		-- get the host id. 
		select into thisHostID id FROM host WHERE name = thisHostName;

		-- if we can not find the host, then we are forced to drop this event into the bit bucket
		if thisHostID is null 
		then
			return -1;
		end if;

		-- get the service id. If this is a new service for this host, insert it. Otherwise, update.
		select into thisServiceID id from service where service_description = ServiceDescription and hostid = thisHostID;

		if thisServiceID is null then
			insert into service (hostid,
					     service_description,
					     active_checks_enabled,
					     passive_checks_enabled,
					     event_handler_enabled,
					     flap_detection_enabled,
					     notifications_enabled,
					     failure_prediction_enabled,
					     process_performance_data,
					     obsess_over_service,
					     event_handler,
					     modified_attributes,
					     has_been_checked,
					     should_be_scheduled)
				values (thisHostID,
					ServiceDescription,
					ActiveChecksEnabled,
					PassiveChecksEnabled,
					EventHandlerEnabled,
					FlapDetectionEnabled,
					NotificationsEnabled,
					FailurePredictionEnabled,
					ProcessPerformanceData,
					ObsessOverService,
					EventHandler,
					ModifiedAttributes,
					HasBeenChecked,
					ShouldBeScheduled);

			-- assume for the moment that this worked just fine
			thisServiceID := currval(''public.service_id_seq'');
		else
			-- okay, the service is already here, so just update it
			update service set hostid = thisHostID,
					active_checks_enabled = ActiveChecksEnabled,
					passive_checks_enabled = PassiveChecksEnabled,
					event_handler_enabled = EventHandlerEnabled,
					flap_detection_enabled = FlapDetectionEnabled,
					notifications_enabled = NotificationsEnabled,
					failure_prediction_enabled = FailurePredictionEnabled,
					process_performance_data = ProcessPerformanceData,
					obsess_over_service = ObsessOverService,
					event_handler = EventHandler,
					modified_attributes = ModifiedAttributes,
					has_been_checked = HasBeenChecked,
					should_be_scheduled = ShouldBeScheduled
				where id = thisServiceID;
		end if;

		-- update these values about the service-check
		-- (or insert the check if it is missing)
		c := count(*) from service_check where serviceID = thisServiceID;
		
		if c = 0
		then
			insert into service_check (check_command,
						check_execute_time,
						check_latency,
						last_check,
						next_check,
						max_attempts,
						last_notification,
						next_notification,
						no_more_notifications,
						performance_data,
						check_type,
						serviceID) 
					values (CheckCommand,
						CheckExecuteTime,
						CheckLatency,
						LastCheck,
						NextCheck,
						MaxAttempts,
						LastNotification,
						NextNotification,
						NoMoreNotifications,
						PerformanceData,
						CheckType,
						thisServiceID);
		else
			update service_check set check_command = CheckCommand,
					      check_execute_time = CheckExecuteTime,
					      check_latency = CheckLatency,
					      last_check = LastCheck,
					      next_check = NextCheck,
					      max_attempts = MaxAttempts,
					      last_notification = LastNotification,
					      next_notification = NextNotification,
					      no_more_notifications = NoMoreNotifications,
					      performance_data = PerformanceData,
					      check_type = CheckType
				where serviceID = thisServiceID;
		end if;


		-- pull this machine in or out of flapping_service based on is_flapping
		-- It will help if we know if we were flapping before this execution
		c := count(*) from flapping_service where id = thisServiceID;

		if IsFlapping = TRUE and c = 0
		then
			-- we are flapping and we were not before
			insert into flapping_service (id) values (thisServiceID);
		else
			if IsFlapping = FALSE and c = 1
			then
				-- we were flapping and we are not anymore
				delete from flapping_service where id = thisServiceID;
			end if;
		end if;

		-- Finally, either extend the current service period or make a new one.
		-- We will need to look up the current soft_state and hard_state.
		select into states soft_state,hard_state FROM service_history WHERE serviceid = thisServiceID and lasted is null;

		-- If they match current_state and last_hard_state, look like we extend.
		if states.soft_state = SoftState and states.hard_state = HardState
		then
			-- Extending means do nothing, unless we are in a soft bad state
			if states.soft_state > 0 -- non-zero means non-good
			then
				-- we need to update:
				-- current_attempt
				update service_history set current_attempt = CurrentAttempt where serviceid = thisServiceID and lasted is null;
			end if;
		else
			-- Otherwise, we end the existing period and start a new one.....
			update service_history set lasted = now()::abstime::int4 - started where serviceid = thisServiceID and lasted is null;
			insert into service_history (started,soft_state,hard_state,plugin_output,serviceid,current_attempt) values
						(now()::abstime::int4,SoftState,HardState,PluginOutput,thisServiceID,CurrentAttempt);
		end if;

		return 0;
        END;
' LANGUAGE 'plpgsql';
